@@ -18,7 +18,7 @@ my $dataset = AI::NNFlex::Dataset->new([
 			[1,0],[1],
 			[1,1],[0]]);
 
-
+my $counter=0;
 my $err = 10;
 while ($err >.01)
 {
@@ -28,7 +28,8 @@ while ($err >.01)
 	# (caught and translated by module)
 	# $err = $network->learn($dataset);
 
-	print "Error = $err\n";
+	print "Epoch $counter: Error = $err\n";
+	$counter++;
 }
 
 
